from typing import Dict

from mythx_models.exceptions import ValidationError
from mythx_models.response.base import BaseResponse


class AuthLogoutResponse(BaseResponse):
    """The API response domain model for a successful logout action."""

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        if not d == {}:
            raise ValidationError(
                "The logout response should be empty but got data: {}".format(d)
            )
        return cls()

    def to_dict(self):
        """Serialize the reponse model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {}
        return d



"""This module contains the base request domain model."""

import abc
import logging

from mythx_models.base import BaseModel

LOGGER = logging.getLogger(__name__)


class BaseRequest(BaseModel, abc.ABC):
    """An abstract object describing requests to the MythX API."""

    @property
    @abc.abstractmethod
    def payload(self):
        """An abstract property returning the request's payload data.

        :return: A Python dict to be serialized into JSON format and submitted to the endpoint.
        """
        pass

    @property
    @abc.abstractmethod
    def headers(self):
        """An abstract property returning additional request headers.

        :return: A dict (str -> str) instance mapping header name to header content
        """
        pass

    @property
    @abc.abstractmethod
    def parameters(self):
        """An abstract property returning additional URL parameters

        :return: A dict (str -> str) instance mapping parameter name to parameter content
        """
        pass

    @property
    @abc.abstractmethod
    def method(self):
        """An abstract property returning the HTTP method to perform.

        :return: The uppercase HTTP method, e.g. "POST"
        """
        pass



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AnalysisSubmissionRequest

from . import common as testdata


def assert_submission_request(req: AnalysisSubmissionRequest):
    assert req.contract_name == testdata.CONTRACT_NAME
    assert req.bytecode == testdata.BYTECODE
    assert req.main_source == testdata.MAIN_SOURCE
    assert req.source_map == testdata.SOURCE_MAP
    assert req.deployed_bytecode == testdata.DEPLOYED_BYTECODE
    assert req.deployed_source_map == testdata.DEPLOYED_SOURCE_MAP
    assert req.sources == testdata.SOURCES
    assert req.solc_version == testdata.SOLC_VERSION
    assert req.analysis_mode == testdata.ANALYSIS_MODE
    assert req.method == "POST"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {"data": testdata.ANALYSIS_SUBMISSION_REQUEST_DICT}


def test_analysis_submission_request_from_valid_json():
    req = AnalysisSubmissionRequest.from_json(
        json.dumps(testdata.ANALYSIS_SUBMISSION_REQUEST_DICT)
    )
    assert_submission_request(req)


def test_analysis_submission_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisSubmissionRequest.from_json("{}")


def test_analysis_submission_request_from_valid_dict():
    req = AnalysisSubmissionRequest.from_dict(testdata.ANALYSIS_SUBMISSION_REQUEST_DICT)
    assert_submission_request(req)


def test_analysis_submission_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisSubmissionRequest.from_dict({})


def test_analysis_submission_request_to_json():
    assert (
        json.loads(testdata.ANALYSIS_SUBMISSION_REQUEST_OBJECT.to_json())
        == testdata.ANALYSIS_SUBMISSION_REQUEST_DICT
    )


def test_analysis_submission_request_to_dict():
    assert (
        testdata.ANALYSIS_SUBMISSION_REQUEST_OBJECT.to_dict()
        == testdata.ANALYSIS_SUBMISSION_REQUEST_DICT
    )


def test_analysis_submission_request_bytecode_only():
    req = AnalysisSubmissionRequest(bytecode=testdata.BYTECODE)
    assert req.bytecode == testdata.BYTECODE
    assert req.analysis_mode == "quick"  # default value
    assert req.to_dict() == {"bytecode": testdata.BYTECODE, "analysisMode": "quick"}
    assert json.loads(req.to_json()) == {
        "bytecode": testdata.BYTECODE,
        "analysisMode": "quick",
    }


def test_analysis_submission_request_source_only():
    req = AnalysisSubmissionRequest(sources=testdata.SOURCES)
    assert req.sources == testdata.SOURCES
    assert req.analysis_mode == "quick"  # default value
    assert req.to_dict() == {"sources": testdata.SOURCES, "analysisMode": "quick"}
    assert json.loads(req.to_json()) == {
        "sources": testdata.SOURCES,
        "analysisMode": "quick",
    }


def test_analysis_submission_request_invalid_mode():
    req = AnalysisSubmissionRequest(bytecode=testdata.BYTECODE, analysis_mode="invalid")
    with pytest.raises(ValidationError):
        req.to_dict()


def test_analysis_submission_request_missing_field():
    req = AnalysisSubmissionRequest()
    with pytest.raises(ValidationError):
        req.to_dict()



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.request import AnalysisStatusRequest

from . import common as testdata


def assert_status_request(req: AnalysisStatusRequest):
    assert req.uuid == testdata.UUID_1
    assert req.method == "GET"
    assert req.headers == {}
    assert req.parameters == {}
    assert req.payload == {}


def test_analysis_status_request_from_valid_json():
    req = AnalysisStatusRequest.from_json(
        json.dumps(testdata.ANALYSIS_STATUS_REQUEST_DICT)
    )
    assert_status_request(req)


def test_analysis_status_request_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisStatusRequest.from_json("{}")


def test_analysis_status_request_from_valid_dict():
    req = AnalysisStatusRequest.from_dict(testdata.ANALYSIS_STATUS_REQUEST_DICT)
    assert_status_request(req)


def test_analysis_status_request_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisStatusRequest.from_dict({})


def test_analysis_status_request_to_json():
    assert (
        json.loads(testdata.ANALYSIS_STATUS_REQUEST_OBJECT.to_json())
        == testdata.ANALYSIS_STATUS_REQUEST_DICT
    )


def test_analysis_status_request_to_dict():
    assert (
        testdata.ANALYSIS_STATUS_REQUEST_OBJECT.to_dict()
        == testdata.ANALYSIS_STATUS_REQUEST_DICT
    )



import json
from pathlib import Path

import dateutil.parser

from mythx_models.request import (
    AnalysisListRequest,
    AnalysisStatusRequest,
    AnalysisSubmissionRequest,
    AuthLoginRequest,
    AuthLogoutRequest,
    AuthRefreshRequest,
    DetectedIssuesRequest,
    OASRequest,
    VersionRequest,
)
from mythx_models.response import (
    Analysis,
    AnalysisListResponse,
    AnalysisStatusResponse,
    AnalysisSubmissionResponse,
    AuthLoginResponse,
    AuthLogoutResponse,
    AuthRefreshResponse,
    DecodedLocation,
    DetectedIssuesResponse,
    Issue,
    IssueReport,
    OASResponse,
    Severity,
    SourceFormat,
    SourceLocation,
    SourceType,
    VersionResponse,
)

# BASE DATA
API_VERSION_1 = "v1.3.0"
API_VERSION_2 = "v1.4.0"
MARU_VERSION_1 = "v0.2.0"
MARU_VERSION_2 = "v0.3.0"
MYTHRIL_VERSION_1 = "0.19.11"
MYTHRIL_VERSION_2 = "0.20.1"
HARVEY_VERSION_1 = "0.0.8"
HARVEY_VERSION_2 = "0.0.9"
HASHED_VERSION_1 = "31337deadbeef"
HASHED_VERSION_2 = "31337cafebabe"

QUEUE_TIME_1 = 1
QUEUE_TIME_2 = 2
RUN_TIME_1 = 300
RUN_TIME_2 = 200

STATUS_1 = "In Progress"
STATUS_2 = "Finished"

SUBMITTED_AT_1 = "2019-01-10T01:29:38.410Z"
SUBMITTED_AT_2 = "2019-01-09T01:29:38.410Z"
SUBMITTED_BY_1 = "000008544b0aa00010a91111"
SUBMITTED_BY_2 = "000008544b0aa00010a91112"

CLIENT_TOOL_NAME_1 = "pythx"
CLIENT_TOOL_NAME_2 = "Edelweiss"  # <3

ERROR = "my test error"

UUID_1 = "0680a1e2-b908-4c9a-a15b-636ef9b61486"
UUID_2 = "0680a1e2-b908-4c9a-a15b-636ef9b61487"

ACCESS_TOKEN_1 = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqdGkiOiIzNmQ0OGI2Yi03ZjFiLTQ0NjEtYjI1OS1hM2M5MmQzMGI4NWQiLCJpc3MiOiJNeXRocmlsIEFQSSIsImV4cCI6MTU1MTAyNTg3OS4wMjgsInVzZXJJZCI6IjEyMzQ1Njc4OTAxMjM0NTY3ODkwMTIzNCIsImlhdCI6MTU1MTAyNTI3OX0.M9SGJayVcreihIFis406wmZLtq3kSsbiV5VAbIuCE0U"
ACCESS_TOKEN_2 = "some other access token"
REFRESH_TOKEN_1 = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqdGkiOiIyNDY3NzZkZC1iY2FjLTQ0NGItOGYwMy02ZDcyYWY3MjcwYWMiLCJpc3MiOiJNeXRocmlsIEFQSSIsImV4cCI6MTU1MzQ0NDQ3OS4wMjgsImlhdCI6MTU1MTAyNTI3OX0.iPRrfFZgnotvMuqu5shi4aQR9GCZR5kUjyzqTTQhZp8"
REFRESH_TOKEN_2 = "some other refresh token"

OFFSET = 0
DATE_FROM = "2019-02-07T00:40:49.058158"
DATE_TO = "2019-02-08T00:40:49.058158"

SWC_ID = "SWC-103"
SWC_TITLE = "Floating Pragma"
DESCRIPTION_HEAD = "A floating pragma is set."
DESCRIPTION_TAIL = 'It is recommended to make a conscious choice on what version of Solidity is used for compilation. Currently any version equal or greater than \\"0.4.24\\" is allowed.'
SEVERITY = "Low"
SOURCE_MAP = "48:1:0"
SOURCE_TYPE = "raw-bytecode"
SOURCE_FORMAT = "evm-byzantium-bytecode"
SOURCE_LIST = "/test/my-super-safe-contract.sol"

CONTRACT_NAME = "PublicStorageArray"
BYTECODE = "60806040526020604051908101604052806000600102815250600090600161002892919061003b565b5034801561003557600080fd5b506100ad565b828054828255906000526020600020908101928215610077579160200282015b8281111561007657825182559160200191906001019061005b565b5b5090506100849190610088565b5090565b6100aa91905b808211156100a657600081600090555060010161008e565b5090565b90565b60dd806100bb6000396000f3fe608060405260043610603f576000357c0100000000000000000000000000000000000000000000000000000000900463ffffffff168063017a9105146044575b600080fd5b348015604f57600080fd5b50607960048036036020811015606457600080fd5b8101908080359060200190929190505050608f565b6040518082815260200191505060405180910390f35b600081815481101515609d57fe5b90600052602060002001600091509050548156fea165627a7a72305820477e0888fcd92cabf059fb331003e0f0bc4dc7d0617752544f0c9a0fc80970aa0029"
SOURCE_MAP = "25:75:0:-;;;59:38;;;;;;;;;94:1;86:10;;59:38;;;;;;;;;;;:::i;:::-;;25:75;8:9:-1;5:2;;;30:1;27;20:12;5:2;25:75:0;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;:::i;:::-;;;:::o;:::-;;;;;;;;;;;;;;;;;;;;;;;;;;;:::o;:::-;;;;;;;"
DEPLOYED_BYTECODE = "608060405260043610603f576000357c0100000000000000000000000000000000000000000000000000000000900463ffffffff168063017a9105146044575b600080fd5b348015604f57600080fd5b50607960048036036020811015606457600080fd5b8101908080359060200190929190505050608f565b6040518082815260200191505060405180910390f35b600081815481101515609d57fe5b90600052602060002001600091509050548156fea165627a7a72305820477e0888fcd92cabf059fb331003e0f0bc4dc7d0617752544f0c9a0fc80970aa0029"
DEPLOYED_SOURCE_MAP = "25:75:0:-;;;;;;;;;;;;;;;;;;;;;;;;59:38;;8:9:-1;5:2;;;30:1;27;20:12;5:2;59:38:0;;;;;;13:2:-1;8:3;5:11;2:2;;;29:1;26;19:12;2:2;59:38:0;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;:::o"
MAIN_SOURCE = "PublicStorageArray.sol"
SOURCES = {
    "PublicStorageArray.sol": {
        "source": "pragma solidity ^0.5.0;\n\ncontract PublicStorageArray {\n    bytes32[] public states = [bytes32(0)];\n}",
        "ast": '{"contracts":{"/tmp/test.sol:PublicStorageArray":{}},"sourceList":["/tmp/test.sol"],"sources":{"/tmp/test.sol":{"AST":{"attributes":{"absolutePath":"/tmp/test.sol","exportedSymbols":{"PublicStorageArray":[9]}},"children":[{"attributes":{"literals":["solidity","^","0.5",".0"]},"id":1,"name":"PragmaDirective","src":"0:23:0"},{"attributes":{"baseContracts":[null],"contractDependencies":[null],"contractKind":"contract","documentation":null,"fullyImplemented":true,"linearizedBaseContracts":[9],"name":"PublicStorageArray","scope":10},"children":[{"attributes":{"constant":false,"name":"states","scope":9,"stateVariable":true,"storageLocation":"default","type":"bytes32[]","visibility":"public"},"children":[{"attributes":{"length":null,"type":"bytes32[]"},"children":[{"attributes":{"name":"bytes32","type":"bytes32"},"id":2,"name":"ElementaryTypeName","src":"59:7:0"}],"id":3,"name":"ArrayTypeName","src":"59:9:0"},{"attributes":{"argumentTypes":null,"isConstant":false,"isInlineArray":true,"isLValue":false,"isPure":true,"lValueRequested":false,"type":"bytes32[1] memory"},"children":[{"attributes":{"argumentTypes":null,"isConstant":false,"isLValue":false,"isPure":true,"isStructConstructorCall":false,"lValueRequested":false,"names":[null],"type":"bytes32","type_conversion":true},"children":[{"attributes":{"argumentTypes":[{"typeIdentifier":"t_rational_0_by_1","typeString":"int_const 0"}],"isConstant":false,"isLValue":false,"isPure":true,"lValueRequested":false,"type":"type(bytes32)","value":"bytes32"},"id":4,"name":"ElementaryTypeNameExpression","src":"86:7:0"},{"attributes":{"argumentTypes":null,"hexvalue":"30","isConstant":false,"isLValue":false,"isPure":true,"lValueRequested":false,"subdenomination":null,"token":"number","type":"int_const 0","value":"0"},"id":5,"name":"Literal","src":"94:1:0"}],"id":6,"name":"FunctionCall","src":"86:10:0"}],"id":7,"name":"TupleExpression","src":"85:12:0"}],"id":8,"name":"VariableDeclaration","src":"59:38:0"}],"id":9,"name":"ContractDefinition","src":"25:75:0"}],"id":10,"name":"SourceUnit","src":"0:102:0"}}},"version":"0.5.0+commit.1d4f565a.Linux.g++"}',
    }
}
SOURCE_LIST = ["PublicStorageArray.sol"]
SOLC_VERSION = "0.5.0+commit.1d4f565a.Linux.g++"
ANALYSIS_MODE = "full"

DECODED_LOCATIONS = [[{"line": 1, "column": 2}, {"line": 3, "column": 4}]]
DECODED_LOCATIONS_OBJ = [
    DecodedLocation(start_line=1, start_column=2, end_line=3, end_column=4)
]

GLOBAL_LOGOUT = False
ETH_ADDRESS = "0x0"
PASSWORD = "supersecure1337"
MOCK_API_URL = "mock://test.com/path"

GROUP_ID_1 = "1680a1e2-b908-4c9a-a15b-636ef9b61487"
GROUP_ID_2 = "2680a1e2-b908-4c9a-a15b-636ef9b61487"
GROUP_NAME = "test"

# ISSUE
SOURCE_LOCATION = {
    "sourceMap": SOURCE_MAP,
    "sourceType": SOURCE_TYPE,
    "sourceFormat": SOURCE_FORMAT,
    "sourceList": SOURCE_LIST,
}
ISSUE_DICT = {
    "swcID": SWC_ID,
    "swcTitle": SWC_TITLE,
    "description": {"head": DESCRIPTION_HEAD, "tail": DESCRIPTION_TAIL},
    "severity": SEVERITY,
    "locations": [SOURCE_LOCATION],
    "decodedLocations": DECODED_LOCATIONS,
    "extra": {},
}
ISSUE_OBJECT = Issue(
    swc_id=SWC_ID,
    swc_title=SWC_TITLE,
    description_short=DESCRIPTION_HEAD,
    description_long=DESCRIPTION_TAIL,
    severity=Severity.LOW,
    locations=[
        SourceLocation(
            source_map=SOURCE_MAP,
            source_type=SourceType.RAW_BYTECODE,
            source_format=SourceFormat.EVM_BYZANTIUM_BYTECODE,
            source_list=SOURCE_LIST,
        )
    ],
    decoded_locations=DECODED_LOCATIONS_OBJ,
    extra={},
)

# ANALYSIS
ANALYSIS_DICT = {
    "apiVersion": API_VERSION_1,
    "maruVersion": MARU_VERSION_1,
    "mythrilVersion": MYTHRIL_VERSION_1,
    "harveyVersion": HARVEY_VERSION_1,
    "queueTime": QUEUE_TIME_1,
    "status": STATUS_1,
    "submittedAt": SUBMITTED_AT_1,
    "submittedBy": SUBMITTED_BY_1,
    "uuid": UUID_1,
    "clientToolName": CLIENT_TOOL_NAME_1,
    "analysisMode": ANALYSIS_MODE,
    "groupName": GROUP_NAME,
    "runTime": 0,
    "groupId": GROUP_ID_1,
}
ANALYSIS_OBJECT = Analysis(
    uuid=UUID_1,
    api_version=API_VERSION_1,
    maru_version=MARU_VERSION_1,
    mythril_version=MYTHRIL_VERSION_1,
    harvey_version=HARVEY_VERSION_1,
    queue_time=QUEUE_TIME_1,
    status=STATUS_1,
    client_tool_name=CLIENT_TOOL_NAME_1,
    submitted_at=SUBMITTED_AT_1,
    submitted_by=SUBMITTED_BY_1,
    group_id=GROUP_ID_1,
    group_name=GROUP_NAME,
    analysis_mode=ANALYSIS_MODE
)

# LOGIN
LOGIN_REQUEST_DICT = {"ethAddress": ETH_ADDRESS, "password": PASSWORD}
LOGIN_REQUEST_OBJECT = AuthLoginRequest(eth_address=ETH_ADDRESS, password=PASSWORD)
LOGIN_RESPONSE_DICT = {
    "jwtTokens": {"access": ACCESS_TOKEN_1, "refresh": REFRESH_TOKEN_1}
}
LOGIN_RESPONSE_OBJECT = AuthLoginResponse(
    access_token=ACCESS_TOKEN_1, refresh_token=REFRESH_TOKEN_1
)

# LOGOUT
LOGOUT_REQUEST_DICT = LOGOUT = {"global": GLOBAL_LOGOUT}
LOGOUT_REQUEST_OBJECT = AuthLogoutRequest()
LOGOUT_RESPONSE_DICT = {}
LOGOUT_RESPONSE_OBJECT = AuthLogoutResponse()

# REFRESH
REFRESH_REQUEST_PAYLOAD_DICT = {
    "jwtTokens": {"access": ACCESS_TOKEN_1, "refresh": REFRESH_TOKEN_1}
}
REFRESH_REQUEST_DICT = LOGIN_RESPONSE_DICT
REFRESH_REQUEST_OBJECT = AuthRefreshRequest(
    access_token=ACCESS_TOKEN_1, refresh_token=REFRESH_TOKEN_1
)
REFRESH_RESPONSE_DICT = LOGIN_RESPONSE_DICT
REFRESH_RESPONSE_OBJECT = AuthRefreshResponse(
    access_token=ACCESS_TOKEN_1, refresh_token=REFRESH_TOKEN_1
)

# OPENAPI
OPENAPI_REQUEST_OBJECT = OASRequest(mode="yaml")
OPENAPI_RESPONSE = "openapi stuff"
OPENAPI_RESPONSE_OBJECT = OASResponse(data=OPENAPI_RESPONSE)

# VERSION
VERSION_REQUEST_OBJECT = VersionRequest()
VERSION_RESPONSE_DICT = {
    "api": API_VERSION_1,
    "maru": MARU_VERSION_1,
    "mythril": MYTHRIL_VERSION_1,
    "harvey": HARVEY_VERSION_1,
    "hash": HASHED_VERSION_1,
}
VERSION_RESPONSE_OBJECT = VersionResponse(
    api_version=API_VERSION_1,
    maru_version=MARU_VERSION_1,
    mythril_version=MYTHRIL_VERSION_1,
    harvey_version=HARVEY_VERSION_1,
    hashed_version=HASHED_VERSION_1,
)

# ANALYSIS SUBMISSION
ANALYSIS_SUBMISSION_REQUEST_DICT = {
    "contractName": CONTRACT_NAME,
    "bytecode": BYTECODE,
    "sourceMap": SOURCE_MAP,
    "deployedBytecode": DEPLOYED_BYTECODE,
    "deployedSourceMap": DEPLOYED_SOURCE_MAP,
    "mainSource": MAIN_SOURCE,
    "sources": SOURCES,
    "sourceList": SOURCE_LIST,
    "version": SOLC_VERSION,
    "analysisMode": ANALYSIS_MODE,
}
ANALYSIS_SUBMISSION_REQUEST_OBJECT = AnalysisSubmissionRequest(
    contract_name=CONTRACT_NAME,
    bytecode=BYTECODE,
    source_map=SOURCE_MAP,
    deployed_bytecode=DEPLOYED_BYTECODE,
    deployed_source_map=DEPLOYED_SOURCE_MAP,
    main_source=MAIN_SOURCE,
    sources=SOURCES,
    source_list=SOURCE_LIST,
    solc_version=SOLC_VERSION,
    analysis_mode=ANALYSIS_MODE,
)
ANALYSIS_SUBMISSION_RESPONSE_DICT = {
    "uuid": UUID_1,
    "apiVersion": API_VERSION_1,
    "maruVersion": MARU_VERSION_1,
    "mythrilVersion": MYTHRIL_VERSION_1,
    "harveyVersion": HARVEY_VERSION_1,
    "queueTime": QUEUE_TIME_1,
    "runTime": RUN_TIME_1,
    "status": STATUS_1,
    "submittedAt": SUBMITTED_AT_1,
    "submittedBy": SUBMITTED_BY_1,
    "clientToolName": CLIENT_TOOL_NAME_1,
    "groupId": GROUP_ID_1,
    "groupName": GROUP_NAME,
    "analysisMode": ANALYSIS_MODE,
}
ANALYSIS_SUBMISSION_RESPONSE_OBJECT = AnalysisSubmissionResponse(
    analysis=Analysis(
        uuid=UUID_1,
        api_version=API_VERSION_1,
        maru_version=MARU_VERSION_1,
        mythril_version=MYTHRIL_VERSION_1,
        harvey_version=HARVEY_VERSION_1,
        queue_time=QUEUE_TIME_1,
        run_time=RUN_TIME_1,
        status=STATUS_1,
        submitted_at=SUBMITTED_AT_1,
        submitted_by=SUBMITTED_BY_1,
        client_tool_name=CLIENT_TOOL_NAME_1,
        group_id=GROUP_ID_1,
        analysis_mode=ANALYSIS_MODE,
        group_name=GROUP_NAME,
    )
)

# ANALYSIS STATUS
ANALYSIS_STATUS_REQUEST_DICT = {"uuid": UUID_1}
ANALYSIS_STATUS_REQUEST_OBJECT = AnalysisStatusRequest(uuid=UUID_1)
ANALYSIS_STATUS_RESPONSE_DICT = ANALYSIS_SUBMISSION_RESPONSE_DICT
ANALYSIS_STATUS_RESPONSE_OBJECT = AnalysisStatusResponse(
    analysis=Analysis(
        uuid=UUID_1,
        api_version=API_VERSION_1,
        maru_version=MARU_VERSION_1,
        mythril_version=MYTHRIL_VERSION_1,
        harvey_version=HARVEY_VERSION_1,
        queue_time=QUEUE_TIME_1,
        run_time=RUN_TIME_1,
        status=STATUS_1,
        submitted_at=SUBMITTED_AT_1,
        submitted_by=SUBMITTED_BY_1,
        client_tool_name=CLIENT_TOOL_NAME_1,
        group_id=GROUP_ID_1,
        group_name=GROUP_NAME,
        analysis_mode=ANALYSIS_MODE,
    )
)

# ANALYSIS LIST
ANALYSIS_LIST_REQUEST_DICT = {
    "offset": OFFSET,
    "dateFrom": DATE_FROM,
    "dateTo": DATE_TO,
}
ANALYSIS_LIST_REQUEST_OBJECT = AnalysisListRequest(
    offset=OFFSET,
    date_from=dateutil.parser.parse(DATE_FROM),
    date_to=dateutil.parser.parse(DATE_TO),
)
ANALYSIS_LIST_RESPONSE_DICT = {
    "analyses": [
        {
            "uuid": UUID_1,
            "apiVersion": API_VERSION_1,
            "maruVersion": MARU_VERSION_1,
            "mythrilVersion": MYTHRIL_VERSION_1,
            "harveyVersion": HARVEY_VERSION_1,
            "queueTime": QUEUE_TIME_1,
            "runTime": RUN_TIME_1,
            "status": STATUS_1,
            "submittedAt": SUBMITTED_AT_1,
            "submittedBy": SUBMITTED_BY_1,
            "clientToolName": CLIENT_TOOL_NAME_1,
            "groupId": GROUP_ID_1,
            "groupName": GROUP_NAME,
            "analysisMode": ANALYSIS_MODE,
        },
        {
            "uuid": UUID_2,
            "apiVersion": API_VERSION_2,
            "maruVersion": MARU_VERSION_2,
            "mythrilVersion": MYTHRIL_VERSION_2,
            "harveyVersion": HARVEY_VERSION_2,
            "queueTime": QUEUE_TIME_2,
            "runTime": RUN_TIME_2,
            "status": STATUS_2,
            "submittedAt": SUBMITTED_AT_2,
            "submittedBy": SUBMITTED_BY_2,
            "clientToolName": CLIENT_TOOL_NAME_2,
            "groupId": GROUP_ID_2,
            "groupName": GROUP_NAME,
            "analysisMode": ANALYSIS_MODE,
        },
    ],
    "total": 2,
}
ANALYSIS_LIST_RESPONSE_OBJECT = AnalysisListResponse(
    analyses=[
        Analysis(
            uuid=UUID_1,
            api_version=API_VERSION_1,
            maru_version=MARU_VERSION_1,
            mythril_version=MYTHRIL_VERSION_1,
            harvey_version=HARVEY_VERSION_1,
            queue_time=QUEUE_TIME_1,
            run_time=RUN_TIME_1,
            status=STATUS_1,
            submitted_at=SUBMITTED_AT_1,
            submitted_by=SUBMITTED_BY_1,
            client_tool_name=CLIENT_TOOL_NAME_1,
            group_id=GROUP_ID_1,
            group_name=GROUP_NAME,
            analysis_mode=ANALYSIS_MODE,
        ),
        Analysis(
            uuid=UUID_2,
            api_version=API_VERSION_2,
            maru_version=MARU_VERSION_2,
            mythril_version=MYTHRIL_VERSION_2,
            harvey_version=HARVEY_VERSION_2,
            queue_time=QUEUE_TIME_2,
            run_time=RUN_TIME_2,
            status=STATUS_2,
            submitted_at=SUBMITTED_AT_2,
            submitted_by=SUBMITTED_BY_2,
            client_tool_name=CLIENT_TOOL_NAME_2,
            group_id=GROUP_ID_2,
            group_name=GROUP_NAME,
            analysis_mode=ANALYSIS_MODE,
        ),
    ],
    total=2,
)

# DETECTED ISSUES
DETECTED_ISSUES_REQUEST_DICT = {"uuid": UUID_1}
DETECTED_ISSUES_REQUEST_OBJECT = DetectedIssuesRequest(uuid=UUID_1)
ISSUE_REPORT_DICT = {
    "issues": [
        {
            "swcID": SWC_ID,
            "swcTitle": SWC_TITLE,
            "description": {"head": DESCRIPTION_HEAD, "tail": DESCRIPTION_TAIL},
            "severity": SEVERITY,
            "locations": [
                {
                    "sourceMap": SOURCE_MAP,
                    "sourceType": SOURCE_TYPE,
                    "sourceFormat": SOURCE_FORMAT,
                    "sourceList": SOURCE_LIST,
                }
            ],
            "decodedLocations": DECODED_LOCATIONS,
            "extra": {},
        }
    ],
    "sourceType": SOURCE_TYPE,
    "sourceFormat": SOURCE_FORMAT,
    "sourceList": SOURCE_LIST,
    "meta": {},
}

ISSUE_REPORT_OBJECT = IssueReport(
    issues=[ISSUE_OBJECT],
    source_type=SourceType.RAW_BYTECODE,
    source_format=SourceFormat.EVM_BYZANTIUM_BYTECODE,
    source_list=SOURCE_LIST,
    meta_data={},
)

DETECTED_ISSUES_RESPONSE_DICT = {"issueReports": [ISSUE_REPORT_DICT]}

DETECTED_ISSUES_RESPONSE_OBJECT = DetectedIssuesResponse(
    issue_reports=[ISSUE_REPORT_OBJECT]
)


def generate_request_dict(req):
    return {
        "method": req.method,
        "payload": req.payload,
        "params": req.parameters,
        "headers": req.headers,
        "url": "https://test.com/" + req.endpoint,
    }


def get_test_case(path):
    with open(str(Path(__file__).parent / path)) as f:
        json_data = f.read()
        dict_data = json.loads(json_data)
    return json_data, dict_data



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import (
    GroupState,
    GroupStatistics,
    GroupStatusResponse,
    VulnerabilityStatistics,
)
from datetime import datetime
from dateutil.tz import tzutc
from .common import get_test_case

JSON_DATA, DICT_DATA = get_test_case("testdata/group-status-response.json")
OBJ_DATA = GroupStatusResponse.from_json(JSON_DATA)


def assert_response(resp: GroupStatusResponse):
    g = resp.group
    assert g.identifier == "test"
    assert g.name == "test"
    assert g.created_at == datetime(2019, 10, 30, 0, 52, 12, tzinfo=tzutc())
    assert g.created_by == "test"
    assert g.completed_at == datetime(2019, 10, 30, 0, 52, 12, tzinfo=tzutc())
    assert g.progress == 0
    assert g.status == GroupState.OPENED
    assert g.main_source_files == ["test"]
    assert g.analysis_statistics == GroupStatistics(
        total=0, queued=0, running=0, failed=0, finished=0
    )
    assert g.vulnerability_statistics == VulnerabilityStatistics(
        high=0, medium=0, low=0, none=0
    )


def test_from_valid_json():
    assert_response(OBJ_DATA)


def test_from_invalid_json():
    with pytest.raises(ValidationError):
        GroupStatusResponse.from_json("{}")


def test_from_valid_dict():
    resp = GroupStatusResponse.from_dict(DICT_DATA)
    assert_response(resp)


def test_from_invalid_dict():
    with pytest.raises(ValidationError):
        GroupStatusResponse.from_dict({})


def test_to_json():
    assert json.loads(OBJ_DATA.to_json()) == DICT_DATA


def test_to_dict():
    assert OBJ_DATA.to_dict() == DICT_DATA



"""This module contains the response models for the detected issues endpoint and a report helper."""
import json
from typing import Any, Dict, List

from mythx_models.base import JSONSerializable
from mythx_models.exceptions import ValidationError
from mythx_models.response.base import BaseResponse
from mythx_models.response.issue import Issue, SourceFormat, SourceType
from mythx_models.util import resolve_schema


class IssueReport(JSONSerializable):
    """The API response domain model for an issues report object."""

    def __init__(
        self,
        issues: List[Issue],
        source_type: SourceType,
        source_format: SourceFormat,
        source_list: List[str],
        meta_data: Dict[str, Any],
    ):
        self.issues = issues
        self.source_type = source_type
        self.source_format = source_format
        self.source_list = source_list
        self.meta_data = meta_data

    @classmethod
    def from_dict(cls, d):
        """Create the issue report domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with  the data from :code:`d` filled in
        """
        return cls(
            issues=[Issue.from_dict(i) for i in d["issues"]],
            source_type=SourceType(d["sourceType"]),
            source_format=SourceFormat(d["sourceFormat"]),
            source_list=d["sourceList"],
            meta_data=d["meta"],
        )

    def to_dict(self):
        """Serialize the issue report domain model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "issues": [i.to_dict() for i in self.issues],
            "sourceType": self.source_type,
            "sourceFormat": self.source_format,
            "sourceList": self.source_list,
            "meta": self.meta_data,
        }
        return d

    def __contains__(self, key: str):
        return any(map(lambda i: i.swc_id == key, self.issues))

    def __len__(self):
        return len(self.issues)

    def __iter__(self):
        for issue in self.issues:
            yield issue

    def __getitem__(self, key):
        return self.issues[key]

    def __setitem__(self, key, value):
        self.issues[key] = value

    def __delitem__(self, key):
        del self.issues[key]

    def __eq__(self, other: "IssueReport"):
        return all((
            self.issues == other.issues,
            self.source_type == other.source_type,
            self.source_format == other.source_format,
            self.source_list == other.source_list,
            self.meta_data == other.meta_data,
        ))


class DetectedIssuesResponse(BaseResponse):
    """The API response domain model for a report of the detected issues."""

    with open(resolve_schema(__file__, "detected-issues.json")) as sf:
        schema = json.load(sf)

    def __init__(self, issue_reports: List[IssueReport]) -> None:
        self.issue_reports = issue_reports

    @classmethod
    def from_dict(cls, d: Dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The List to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """

        if type(d) == list:
            cls.validate(d)
            d = {"issueReports": d}
        elif type(d) == dict:
            if d.get("issueReports") is None:
                raise ValidationError(
                    "Cannot create DetectedIssuesResponse object from invalid dictionary d: {}".format(
                        d
                    )
                )

            cls.validate(d["issueReports"])
        else:
            raise ValidationError(
                "Expected list or dict but got {} of type {}".format(d, type(d))
            )

        return cls(issue_reports=[IssueReport.from_dict(i) for i in d["issueReports"]])

    def to_dict(self, as_list=False):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"issueReports": [report.to_dict() for report in self.issue_reports]}
        self.validate(d["issueReports"])
        return d["issueReports"] if as_list else d

    def to_json(self):
        """Serialize the model to JSON format.

        Internally, this method is using the :code:`to_dict` method.

        :return: A JSON string holding the model's data
        """
        return json.dumps([report.to_dict() for report in self.issue_reports])

    def __contains__(self, key: str):
        if not type(key) == str:
            raise ValueError(
                "Expected SWC ID of type str but got {} of type {}".format(
                    key, type(key)
                )
            )
        for report in self.issue_reports:
            if key in report:
                return True
        return False

    def __iter__(self):
        for report in self.issue_reports:
            for issue in report:
                yield issue

    def __len__(self):
        total_detected_issues = 0
        for report in self.issue_reports:
            total_detected_issues += len(report)
        return total_detected_issues

    def __getitem__(self, key: int):
        return self.issue_reports[key]

    def __setitem__(self, key: int, value: IssueReport):
        self.issue_reports[key] = value

    def __delitem__(self, key: int):
        del self.issue_reports[key]

    def __eq__(self, other: "DetectedIssuesResponse"):
        return self.issue_reports == other.issue_reports



import json
from copy import deepcopy

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import AnalysisListResponse
from mythx_models.response.analysis import Analysis
from mythx_models.util import serialize_api_timestamp

from . import common as testdata


def assert_analysis_data(expected, analysis: Analysis):
    assert expected["apiVersion"] == analysis.api_version
    assert expected["maruVersion"] == analysis.maru_version
    assert expected["mythrilVersion"] == analysis.mythril_version
    assert expected["harveyVersion"] == analysis.harvey_version
    assert expected["queueTime"] == analysis.queue_time
    assert expected["runTime"] == analysis.run_time
    assert expected["status"] == analysis.status
    assert expected["submittedAt"] == serialize_api_timestamp(analysis.submitted_at)
    assert expected["submittedBy"] == analysis.submitted_by
    assert expected["uuid"] == analysis.uuid


def test_analysis_list_from_valid_json():
    resp = AnalysisListResponse.from_json(
        json.dumps(testdata.ANALYSIS_LIST_RESPONSE_DICT)
    )
    assert len(resp.analyses) == 2
    for i, analysis in enumerate(resp.analyses):
        assert_analysis_data(
            testdata.ANALYSIS_LIST_RESPONSE_DICT["analyses"][i], analysis
        )


def test_analysis_list_from_invalid_json():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_json("[]")


def test_analysis_list_from_empty_json():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_json("{}")


def test_analysis_list_from_valid_dict():
    resp = AnalysisListResponse.from_dict(testdata.ANALYSIS_LIST_RESPONSE_DICT)
    assert len(resp.analyses) == 2
    for i, analysis in enumerate(resp.analyses):
        assert_analysis_data(
            testdata.ANALYSIS_LIST_RESPONSE_DICT["analyses"][i], analysis
        )


def test_analysis_list_from_invalid_dict():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_dict("[]")


def test_analysis_list_from_empty_dict():
    with pytest.raises(ValidationError):
        AnalysisListResponse.from_dict({})


def test_analysis_list_to_dict():
    d = testdata.ANALYSIS_LIST_RESPONSE_OBJECT.to_dict()
    assert d == testdata.ANALYSIS_LIST_RESPONSE_DICT


def test_analysis_list_to_json():
    json_str = testdata.ANALYSIS_LIST_RESPONSE_OBJECT.to_json()
    assert json.loads(json_str) == testdata.ANALYSIS_LIST_RESPONSE_DICT


def test_iteration():
    uuids = (testdata.UUID_1, testdata.UUID_2)
    # XXX: Don't use zip here to make sure __iter__ returns
    for idx, analysis in enumerate(testdata.ANALYSIS_LIST_RESPONSE_OBJECT):
        assert uuids[idx] == analysis.uuid


def test_valid_getitem():
    for idx, analysis in list(
        enumerate(testdata.ANALYSIS_LIST_RESPONSE_OBJECT.analyses)
    ):
        assert testdata.ANALYSIS_LIST_RESPONSE_OBJECT[idx] == analysis


def test_invalid_getitem_index():
    with pytest.raises(IndexError):
        testdata.ANALYSIS_LIST_RESPONSE_OBJECT[1337]


def test_oob_getitem_slice():
    testdata.ANALYSIS_LIST_RESPONSE_OBJECT[1337:9001] == []


def test_valid_delitem():
    analysis_list = deepcopy(testdata.ANALYSIS_LIST_RESPONSE_OBJECT)
    del analysis_list[0]
    assert analysis_list.analyses == testdata.ANALYSIS_LIST_RESPONSE_OBJECT.analyses[1:]
    assert analysis_list.total == testdata.ANALYSIS_LIST_RESPONSE_OBJECT.total - 1
    assert len(analysis_list) == testdata.ANALYSIS_LIST_RESPONSE_OBJECT.total - 1


def test_invalid_delitem():
    with pytest.raises(IndexError):
        del testdata.ANALYSIS_LIST_RESPONSE_OBJECT[1337]


def test_valid_setitem():
    analysis_list = deepcopy(testdata.ANALYSIS_LIST_RESPONSE_OBJECT)
    analysis_list[0] = "foo"
    assert analysis_list.analyses[0] == "foo"
    assert analysis_list[0] == "foo"
    assert len(analysis_list.analyses) == len(
        testdata.ANALYSIS_LIST_RESPONSE_OBJECT.analyses
    )


def test_invalid_setitem():
    with pytest.raises(IndexError):
        testdata.ANALYSIS_LIST_RESPONSE_OBJECT[1337] = "foo"


def test_reversed():
    analysis_list = deepcopy(testdata.ANALYSIS_LIST_RESPONSE_OBJECT)
    assert list(reversed(analysis_list)) == list(
        reversed(testdata.ANALYSIS_LIST_RESPONSE_OBJECT.analyses)
    )
    assert analysis_list.total == testdata.ANALYSIS_LIST_RESPONSE_OBJECT.total


def test_valid_object_contains():
    assert testdata.ANALYSIS_OBJECT in testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_valid_uuid_contains():
    assert testdata.ANALYSIS_OBJECT.uuid in testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_contains_error():
    with pytest.raises(ValueError):
        1 in testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_invalid_object_contains():
    analysis = deepcopy(testdata.ANALYSIS_OBJECT)
    analysis.uuid = "something else"
    assert not analysis in testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_invalid_uuid_contains():
    assert not "foo" in testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_list_not_equals():
    analysis_list = deepcopy(testdata.ANALYSIS_LIST_RESPONSE_OBJECT)
    analysis_list.total = 0
    assert analysis_list != testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_nested_analysis_not_equals():
    analysis_list = deepcopy(testdata.ANALYSIS_LIST_RESPONSE_OBJECT)
    analysis_list.analyses[0].uuid = "foo"
    assert analysis_list != testdata.ANALYSIS_LIST_RESPONSE_OBJECT


def test_valid_equals():
    analysis_list = deepcopy(testdata.ANALYSIS_LIST_RESPONSE_OBJECT)
    assert analysis_list == testdata.ANALYSIS_LIST_RESPONSE_OBJECT



import json
from typing import List

from mythx_models.exceptions import ValidationError
from mythx_models.response.base import BaseResponse
from mythx_models.response.group import Group
from mythx_models.util import resolve_schema

INDEX_ERROR_MSG = "Group at index {} was not fetched"


class GroupListResponse(BaseResponse):
    """The API response domain model for a list of analyses."""

    with open(resolve_schema(__file__, "group-list.json")) as sf:
        schema = json.load(sf)

    def __init__(self, groups: List[Group], total: int) -> None:
        self.groups = groups
        self.total = total

    @classmethod
    def from_dict(cls, d: dict):
        """Create the response domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        cls.validate(d)
        groups = [Group.from_dict(a) for a in d["groups"]]
        return cls(groups=groups, total=d["total"])

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"groups": [a.to_dict() for a in self.groups], "total": len(self.groups)}
        self.validate(d)
        return d

    def __iter__(self):
        for analysis in self.groups:
            yield analysis

    def __getitem__(self, idx):
        try:
            return self.groups[idx]
        except IndexError:
            raise IndexError(INDEX_ERROR_MSG.format(idx))

    def __setitem__(self, idx, value):
        try:
            self.groups[idx] = value
        except IndexError:
            raise IndexError(INDEX_ERROR_MSG.format(idx))

    def __delitem__(self, idx):
        try:
            del self.groups[idx]
            self.total -= 1
        except IndexError:
            raise IndexError(INDEX_ERROR_MSG.format(idx))

    def __len__(self):
        return self.total

    def __reversed__(self):
        return reversed(self.groups)

    def __contains__(self, item: Group):
        if not type(item) in (Group, str):
            raise ValueError(
                "Expected type Group or str but got {}".format(type(item))
            )
        identifier = item.identifier if type(item) == Group else item
        return identifier in map(lambda x: x.identifier, self.groups)

    def __eq__(self, candidate):
        return all((
            self.total == candidate.total,
            self.groups == candidate.groups,
        ))



import json

import pytest

from mythx_models.exceptions import ValidationError
from mythx_models.response import Analysis, AnalysisSubmissionResponse
from mythx_models.util import serialize_api_timestamp

from . import common as testdata


def assert_analysis_data(expected, analysis: Analysis):
    assert expected["apiVersion"] == analysis.api_version
    assert expected["maruVersion"] == analysis.maru_version
    assert expected["mythrilVersion"] == analysis.mythril_version
    assert expected["harveyVersion"] == analysis.harvey_version
    assert expected["queueTime"] == analysis.queue_time
    assert expected["runTime"] == analysis.run_time
    assert expected["status"] == analysis.status
    assert expected["submittedAt"] == serialize_api_timestamp(analysis.submitted_at)
    assert expected["submittedBy"] == analysis.submitted_by
    assert expected["uuid"] == analysis.uuid


def test_analysis_submission_from_valid_json():
    resp = AnalysisSubmissionResponse.from_json(
        json.dumps(testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT)
    )
    assert_analysis_data(testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT, resp.analysis)


def test_analysis_submission_from_empty_json():
    with pytest.raises(ValidationError):
        AnalysisSubmissionResponse.from_json("{}")


def test_analysis_submission_from_valid_dict():
    resp = AnalysisSubmissionResponse.from_dict(
        testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT
    )
    assert_analysis_data(testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT, resp.analysis)


def test_analysis_submission_from_empty_dict():
    with pytest.raises(ValidationError):
        AnalysisSubmissionResponse.from_dict({})


def test_analysis_submission_to_dict():
    d = testdata.ANALYSIS_SUBMISSION_RESPONSE_OBJECT.to_dict()
    assert d == testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT


def test_analysis_submission_to_json():
    json_str = testdata.ANALYSIS_SUBMISSION_RESPONSE_OBJECT.to_json()
    assert json.loads(json_str) == testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT


def test_analysis_submission_property_delegation():
    assert_analysis_data(
        testdata.ANALYSIS_SUBMISSION_RESPONSE_DICT,
        testdata.ANALYSIS_SUBMISSION_RESPONSE_OBJECT,
    )



"""This module contains domain models regarding analysis jobs"""

import logging
from enum import Enum
from typing import Dict, List

from inflection import underscore

from mythx_models.response.base import BaseResponse
from mythx_models.util import deserialize_api_timestamp, serialize_api_timestamp

LOGGER = logging.getLogger(__name__)


class GroupStatistics(BaseResponse):
    """A container class holding data about a group's analysis jobs"""

    def __init__(
        self,
        total: int,
        queued: int,
        running: int,
        failed: int,
        finished: int,
        *args,
        **kwargs
    ):
        self.total = total
        self.queued = queued
        self.running = running
        self.failed = failed
        self.finished = finished

        if args or kwargs:
            LOGGER.warning(
                "Got unexpected arguments args={}, kwargs={}".format(args, kwargs)
            )

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        d = {k: v for k, v in d.items()}
        return cls(**d)

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "total": self.total,
            "queued": self.queued,
            "running": self.running,
            "failed": self.failed,
            "finished": self.finished,
        }
        return d

    def __eq__(self, other: "GroupStatistics"):
        return all((
            self.total == other.total,
            self.queued == other.queued,
            self.running == other.running,
            self.failed == other.failed,
            self.finished == other.finished
        ))


class VulnerabilityStatistics(BaseResponse):
    """A container class holding data about a group's vulnerabilities"""

    def __init__(self, high: int, medium: int, low: int, none: int, *args, **kwargs):
        self.high = high
        self.medium = medium
        self.low = low
        self.none = none

        if args or kwargs:
            LOGGER.warning(
                "Got unexpected arguments args={}, kwargs={}".format(args, kwargs)
            )

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        d = {k: v for k, v in d.items()}
        return cls(**d)

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {"high": self.high, "medium": self.medium, "low": self.low, "none": self.none}
        return d

    def __eq__(self, other: "VulnerabilityStatistics"):
        return all((
            self.high == other.high,
            self.medium == other.medium,
            self.low == other.low,
            self.none == other.none,
        ))


class GroupState(str, Enum):
    """An Enum describing the status of an analysis group."""

    OPENED = "opened"
    SEALED = "sealed"


class Group(BaseResponse):
    """An object describing an analysis group.

    Such a model was built, because many other API responses deliver the same data when it comes
    to analysis groups. This makes the code more DRY, validation easier, and allows for recursive
    SerDe (e.g. mapping :code:`from_dict` to a deserialized JSON list of job objects.
    """

    def __init__(
        self,
        identifier: str,
        name: str,
        created_at: str,
        created_by: str,
        completed_at: str,
        progress: int,
        status: GroupState,
        main_source_files: List[str],
        analysis_statistics: Dict[str, int],
        vulnerability_statistics: Dict[str, int],
        *args,
        **kwargs
    ):
        self.identifier = identifier
        self.name = name
        self.created_at = deserialize_api_timestamp(created_at)
        self.created_by = created_by
        self.completed_at = deserialize_api_timestamp(completed_at)
        self.progress = progress
        self.main_source_files = main_source_files
        self.status = GroupState(status.lower())
        self.analysis_statistics = GroupStatistics.from_dict(analysis_statistics)
        self.vulnerability_statistics = VulnerabilityStatistics.from_dict(
            vulnerability_statistics
        )

        if args or kwargs:
            LOGGER.warning(
                "Got unexpected arguments args={}, kwargs={}".format(args, kwargs)
            )

    @classmethod
    def from_dict(cls, d):
        """Create the response domain model from a dict.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        d = {underscore(k): v for k, v in d.items()}
        d["identifier"] = d.pop("id")
        d["vulnerability_statistics"] = d.pop("num_vulnerabilities")
        d["analysis_statistics"] = d.pop("num_analyses")
        return cls(**d)

    def to_dict(self):
        """Serialize the response model to a Python dict.

        :return: A dict holding the request model data
        """
        d = {
            "id": self.identifier,
            "name": self.name,
            "createdAt": serialize_api_timestamp(self.created_at),
            "createdBy": self.created_by,
            "completedAt": serialize_api_timestamp(self.completed_at),
            "progress": self.progress,
            "mainSourceFiles": self.main_source_files,
            "status": self.status.lower(),
            "numAnalyses": self.analysis_statistics.to_dict(),
            "numVulnerabilities": self.vulnerability_statistics.to_dict(),
        }
        return d

    def __eq__(self, other: "Group"):
        return all((
            self.identifier == other.identifier,
            self.name == other.name,
            self.created_at == other.created_at,
            self.created_by == other.created_by,
            self.completed_at == other.completed_at,
            self.progress == other.progress,
            self.main_source_files == other.main_source_files,
            self.status == other.status,
            self.analysis_statistics == other.analysis_statistics,
            self.vulnerability_statistics == other.vulnerability_statistics,
        ))

    def __repr__(self):
        return "<Group id={} name={}>".format(self.identifier, self.name)



from mythx_models.response.group_status import GroupStatusResponse


class GroupCreationResponse(GroupStatusResponse):
    """The API response domain model for the status of a newly created group."""

    pass



"""This module contains the GroupListRequest domain model."""

from datetime import datetime
from typing import Any, Dict

import dateutil.parser

from mythx_models.exceptions import ValidationError
from mythx_models.request.base import BaseRequest

GROUP_LIST_KEYS = ("offset", "createdBy", "groupName", "dateFrom", "dateTo")


class GroupListRequest(BaseRequest):
    """Perform an API request that lists the logged in user's past analyses."""

    def __init__(
        self,
        offset: int,
        created_by: str,
        group_name: str,
        date_from: datetime,
        date_to: datetime,
    ):
        self.offset = offset
        self.created_by = created_by
        self.group_name = group_name
        self.date_from = date_from
        self.date_to = date_to

    @property
    def endpoint(self):
        """The API's group list endpoint.

        :return: A string denoting the list endpoint without the host prefix
        """
        return "v1/analysis-groups"

    @property
    def method(self):
        """The HTTP method to perform.

        :return: The uppercase HTTP method, e.g. "POST"
        """
        return "GET"

    @property
    def headers(self):
        """Additional request headers.

        :return: A dict (str -> str) instance mapping header name to header content
        """
        return {}

    @property
    def parameters(self):
        """Additional URL parameters

        :return: A dict (str -> str) instance mapping parameter name to parameter content
        """
        return self.to_dict()

    @property
    def payload(self):
        """The request's payload data.

        :return: A Python dict to be serialized into JSON format and submitted to the endpoint.
        """
        return {}

    @classmethod
    def from_dict(cls, d: Dict[str, Any]):
        """Create the request domain model from a dict.

        This also validates the dict's schema and raises a :code:`ValidationError`
        if any required keys are missing or the data is malformed.

        :param d: The dict to deserialize from
        :return: The domain model with the data from :code:`d` filled in
        """
        if not all(k in d for k in GROUP_LIST_KEYS):
            raise ValidationError(
                "Not all required keys {} found in data {}".format(GROUP_LIST_KEYS, d)
            )
        # TODO: Validate whether offset valid integer
        req = cls(
            offset=d["offset"],
            created_by=d["createdBy"],
            group_name=d["groupName"],
            date_from=dateutil.parser.parse(d["dateFrom"]),
            date_to=dateutil.parser.parse(d["dateTo"]),
        )

        return req

    def to_dict(self):
        """Serialize the request model to a Python dict.

        :return: A dict holding the request model data
        """
        return {
            "offset": self.offset,
            "createdBy": self.created_by,
            "groupName": self.group_name,
            "dateFrom": self.date_from.isoformat() if self.date_from else None,
            "dateTo": self.date_to.isoformat() if self.date_to else None,
        }

    def __eq__(self, other: "GroupListRequest"):
        return all((
            self.offset == other.offset,
            self.created_by == other.created_by,
            self.group_name == other.group_name,
            self.date_from == other.date_from,
            self.date_to == other.date_to,
        ))



"""This module contains various utility functions for MythX domain models."""

from datetime import datetime
from os import path
from typing import Dict

import dateutil.parser


def deserialize_api_timestamp(timestamp_str: str):
    """Deserialize a JavaScript API timestand into Python datetime format.

    :param timestamp_str: The JS timestamp, e.g. :code:`2019-01-10T01:29:38.410Z`
    :return: A Python datetime object
    """
    return dateutil.parser.parse(timestamp_str)


def serialize_api_timestamp(ts_obj: datetime):
    """Serialize a Python datetime object to its JS equivalent.

    :param ts_obj: A Python datetime object
    :return: The JS timestamp, e.g. :code:`2019-01-10T01:29:38.410Z`
    """
    ts_str = ts_obj.strftime("%Y-%m-%dT%H:%M:%S.%f")
    # chop off last 3 digits because JS
    return ts_str[:-3] + "Z"


def dict_delete_none_fields(d: Dict):
    """Remove all keys that have "None" values in a dict.

    :param d: The dictionary to sanitize
    :return: The dict instance with all "None keys" removed
    """
    for k, v in list(d.items()):
        if v is None:
            del d[k]
        elif isinstance(v, dict):
            dict_delete_none_fields(v)
    return d


def resolve_schema(module_path, filename):
    """Return a path leading to the internal JSON schema files used for validation.

    :param module_path: The calling module's path (used as base path)
    :param filename: The JSON schema file's name
    :return: The complete path leading to the schema file (e.g. to be consumed by :code:`open()`)
    """
    return path.join(path.dirname(module_path), "schema", filename)
